# Implementation of three Improved Generalization Bounds for Meta-Learning

we build our project based on the implementation of the paper Tianyu Liu, Jie Lu, Zheng Yan, Guangquan Zhang, “PAC-Bayes Bounds for Meta-learning with Data-Dependent Prior”,  Arxiv 2021 
[https://github.com/tyliu22/Meta-learning-PAC-Bayes-bound-with-data-depedent-prior]

## Data

The data sets are downloaded automatically. Specify the main data path in the file 'Data_Path.py'

## Reproducing experiments in our paper:

python PriorMetaLearning/run_MPB_ShuffledPixels.py   -- Run our PAC-Bayes bound-minimization algorthim in shuffled pixel environment. 
python PriorMetaLearning/run_MPB_PermuteLabels.py    -- Run our PAC-Bayes bound-minimization algorthim in permuted label environment. 

-- When choose the hyperparameter 'complexity-type' as 'PAC_Bayes_improved_classic' / 'PAC_Bayes_improved_quad' / 'PAC_Bayes_improved_lambda' , it runs our classic/quadratic/lambda bound-minimization algorithm

python PriorMetaLearning/run_MPB_ShuffledPixels_TasksN_100_pixels.py - Run our lambda bound-minimization algorithm across a range of the number $n$ of training tasks in the 100-pixel-shuffled environment

python PriorMetaLearning/run_MPB_ShuffledPixels_SampleM_100_pixels.py -Run our lambda bound-minimization algorithm across a range of the sample size $m$ per task in the 100-pixel-shuffled environment


- 'PriorMetaLearning'
   to analyze the influence of different PAC-Bayes bounds with permuted labels and pixels
   - 'NewBoundMcAllaster'
   - 'NewBoundSeeger'
   - 'PAC_Bayes_lambda'
   - 'PAC_Bayes_quad' 
   - 'PAC_Bayes_variational_role'
   ****** Our Improved Three Bounds ******
   - 'PAC_Bayes_improved_classic'
   - 'PAC_Bayes_improved_quad'
   - 'PAC_Bayes_improved_lambda'
   
 ## Project Structure
```
.
|----Data_Path.py  
|----data/
|----Models/
|    |----deterministic_models.py  
|    |----layer_inits.py  
|    |----stochastic_inits.py  
|    |----stochastic_layers.py  
|    |----stochastic_models.py  
|----PriorMetaLearning/
|    |----Analyze_Prior.py  
|    |----AvargeTransfer.py  
|    |----Get_Objective_MPB.py  
|    |----main_Meta_Bayes.py  
|    |----meta_test_Bayes.py  
|    |----meta_train_Bayes_finite_tasks.py  
|    |----meta_train_Bayes_infinite_tasks.py  
|    |----run_MPB_PermutedLabels_TasksN.py  
|    |----run_MPB_PermuteLabels.py  
|    |----run_MPB_ShuffledPixels.py  
|    |----run_MPB_ShuffledPixels_TasksN.py  
|    |----run_MPB_ShuffledPixels_SamplesM.py
|----figure/
|    |----Comparisons_with_other_Three_Methods/
|    |----Convergence_of_Our_Three_bounds/
|----Utils/
|    |----Bayes_utils.py  
|    |----common.py  
|    |----data_gen.py  
|    |----DP_data_gen.py  
|    |----imagenet_data.py  
|    |----omniglot.py  
|    |----Resize_ImageNet.py  
|----README.md  
|----requirements.txt  
```